@@ -84,6 +84,11 @@
   :group 'ess-S
   :prefix "ess-")
 
+(defgroup ess-Julia nil
+  "ESS: Julia."
+  :group 'ess
+  :prefix "julia-")
+
 (defgroup ess-sas nil
   "ESS: SAS."
   :group 'ess
@@ -128,7 +133,7 @@
   :prefix "ess-")
 ;; Variables (not user-changeable)
 
-(defvar ess-version "12.09-2" ;; updated by 'make'
+(defvar ess-version "13.05" ;; updated by 'make'
   "Version of ESS currently loaded.")
 
 (defvar ess-revision nil ;; set
@@ -351,6 +356,7 @@ for all projects."
   :type '(choice (const :tag "Off" nil)
                  (const :tag "On" t)
                  file))
+(make-variable-buffer-local 'ess-history-file)
 
 (defcustom ess-plain-first-buffername t
   "No fancy process buffname for the first process of each type (novice mode)."
@@ -470,7 +476,7 @@ to install your custom sources.
   :group 'R
   :type 'string)
 
-(defcustom ess-use-tracebug nil
+(defcustom ess-use-tracebug t
   "If t, load ess-tracebug when R process starts."
   :group 'ess-extras
   :type  'boolean)
@@ -513,10 +519,25 @@ buffer or end chunks etc.
 
 (defcustom ess-S-assign " <- "
   "String used for left assignment in all S dialects.
- Used by \\[ess-smart-underscore]."
+ Used by \\[ess-smart-S-assign]."
   :group 'ess-S
   :type 'string)
 
+(defcustom ess-smart-S-assign-key "_"
+  "Key used by `ess-smart-S-assign'. By default bound to
+underscore, but can be set to any key. If this key is customized,
+you must add
+
+ (ess-toggle-S-assign nil)
+ (ess-toggle-S-assign nil)
+
+after the line that sets the customization and evaluate these
+lines or reboot emacs. The first call clears the default
+`ess-smart-S-assign' assignment and the second line re-assigns
+it to the customized setting. "
+  :group 'ess-S
+  :type 'character)
+
 ;;*;; Variables concerning editing behaviour
 
 (defcustom ess-filenames-map t
@@ -798,10 +819,11 @@ these values, use the customize interface."
          (set symbol value)
          (ess-add-style 'OWN value)))
 
-(defcustom ess-default-style 'DEFAULT
+(defcustom ess-default-style 'RRR
   "The default value of `ess-style'.
-See the variable `ess-style-alist' for how these groups (DEFAULT,
-OWN, GNU, BSD, ...) map onto different settings for variables."
+See the variable `ess-style-alist' for how these groups (RRR, DEFAULT,
+OWN, GNU, BSD, ...) map onto different settings for variables.
+Since ESS 13.05, the default is  RRR  rather than DEFAULT."
   :type '(choice (const DEFAULT)
                  (const OWN)
                  (const GNU)
@@ -913,6 +935,10 @@ Good for evaluating ESS code."
   :type 'hook
   :group 'ess-R)
 
+(defcustom SAS-mode-hook nil
+  "Hook to run when entering SAS mode."
+  :type 'hook
+  :group 'ess-sas)
 
 (defcustom ess-pdf-viewer-pref nil
   "External pdf viewer you like to use from ESS.
@@ -1614,7 +1640,7 @@ The other variables ...-program-name should be changed, for the
 corresponding program.")
 
 (make-variable-buffer-local 'inferior-ess-program)
-(setq-default inferior-ess-program inferior-S-program-name)
+;; (setq-default inferior-ess-program inferior-S-program-name)
 
 
 (defvar inferior-R-version "R (newest)"
@@ -1655,6 +1681,9 @@ Set to nil if language doesn't support secondary prompt.")
 (make-variable-buffer-local 'inferior-ess-secondary-prompt)
 ;; (setq-default inferior-ess-secondary-prompt "+ ")
 
+(defvar ess-traceback-command nil
+  "Command to generate error traceback.")
+
 ;; need to recognise  + + + > > >
 ;; and "+ . + " in tracebug prompt
 (defcustom inferior-S-prompt "[]a-zA-Z0-9.[]*\\([>+.] \\)*> "
@@ -1804,7 +1833,7 @@ This format string should use %s to substitute an object name.")
 (setq-default inferior-ess-help-command "help(\"%s\")\n")
 
 
-(defcustom inferior-ess-r-help-command ".help.ESS(\"%s\", help_type=\"text\")\n"
+(defcustom inferior-ess-r-help-command ".ess_help(\"%s\", help_type=\"text\")\n"
   "Format-string for building the R command to ask for help on an object.
 
 This format string should use %s to substitute an object name.
@@ -1844,7 +1873,8 @@ Really set in <ess-lang>-customize-alist in ess[dl]-*.el")
 ;;   :group 'ess-command
 ;;   :type 'string)
 
-(defcustom inferior-ess-safe-names-command "try(print(names(%s), max=1e6), silent=TRUE)\n"
+(defcustom inferior-ess-safe-names-command
+  "tryCatch(base::print(base::names(%s), max=1e6), error=function(e){})\n"
   "Format string for ESS command to extract names from an object *safely*.
 
 %s is replaced by an \"object name\" -- usually a list or data frame, but in R also
@@ -2401,10 +2431,14 @@ Passed to `ess-execute-dialect-specific' which see. ")
 
 (defvar ess-funargs-command  nil
   "Dialect specific command to return a list of function arguments.
-See `ess-function-arguments' and .ess.funargs command in R and
+See `ess-function-arguments' and .ess_funargs command in R and
 S+ for details of the format that should be returned.")
 (make-variable-buffer-local 'ess-funargs-command)
 
+(defvar ess-eldoc-function nil
+  "Holds a dialect specific eldoc function,
+See `ess-R-eldoc-function' and `ess-julia-eldoc-function' for examples.")
+
 (defcustom ess-r-args-noargsmsg "No args found."
   "Message returned if \\[ess-r-args-get] cannot find a list of arguments."
   :group 'ess-R
@@ -2455,10 +2489,15 @@ Defaults to `ess-S-non-functions'."
 
 
  ; julia-mode
-(defvar inferior-julia-program-name "julia-release-basic"
+(defcustom inferior-julia-program-name "julia-release-basic"
   ;; the default assumes it is on the PATH ... which is typically the case after
   ;; a "typical unix-alike installation"
-  "Path to julia-release-basic executable")
+  "Path to julia-release-basic executable"
+  :group 'ess-Julia)
+
+(defvar julia-basic-offset 4
+  "Offset for julia code editing")
+
 
 
  ; ess-mode: editing S source
